@@ -164,7 +164,6 @@ from xpra.keys import mask_to_names, DEFAULT_MODIFIER_MEANINGS, DEFAULT_MODIFIER
 from xpra.window_backing import new_backing
 from xpra.platform.gui import ClientExtras
 from xpra.scripts.main import ENCODINGS
-from xpra.version_util import is_compatible_with
 
 if sys.version < '3':
     import codecs
@@ -355,18 +354,26 @@ class ClientWindow(gtk.Window):
         gtk.Window.do_configure_event(self, event)
         if not self._override_redirect:
             x, y, w, h = get_window_geometry(self)
-            if (x, y) != self._pos:
-                ox, oy = self._pos
-                dx, dy = x-ox, y-oy
-                self._pos = (x, y)
-                self._client.send(["move-window", self._id, x, y])
+            ox, oy = self._pos
+            dx, dy = x-ox, y-oy
+            self._pos = (x, y)
+            if self._client.window_configure:
+                #if we support configure-window, send that first
+                self._client.send(["configure-window", self._id, x, y, w, h])
+            if dx!=0 or dy!=0:
+                #window has moved
+                if not self._client.window_configure:
+                    #if we don't handle the move via configure:
+                    self._client.send(["move-window", self._id, x, y])
+                #move any OR window with their parent:
                 for window in self._override_redirect_windows:
                     x, y = window.get_position()
                     window.move(x+dx, y+dy)
             if (w, h) != self._size:
                 self._size = (w, h)
-                self._client.send(["resize-window", self._id, w, h])
                 self.new_backing(w, h)
+                if not self._client.window_configure:
+                    self._client.send(["resize-window", self._id, w, h])
 
     def move_resize(self, x, y, w, h):
         assert self._override_redirect
@@ -467,8 +474,6 @@ class XpraClient(XpraClientBase):
             """ jpegquality was not set, use a better start value """
             self.jpegquality = 50
 
-        self.server_capabilities = {}
-
         self.mmap_enabled = False
         self.server_start_time = -1
         self.server_platform = ""
@@ -484,6 +489,7 @@ class XpraClient(XpraClientBase):
         self.cursors_enabled = True
         self.notifications_enabled = True
         self.clipboard_enabled = False
+        self.window_configure = False
         self.mmap = None
         self.mmap_token = None
         self.mmap_file = None
@@ -603,7 +609,8 @@ class XpraClient(XpraClientBase):
         gtk.main()
         return  self.exit_code
 
-    def quit(self, *args):
+    def quit(self, exit_code=0):
+        self.exit_code = exit_code
         gtk_main_quit_really()
 
     def cleanup(self):
@@ -921,9 +928,8 @@ class XpraClient(XpraClientBase):
         log.debug("Automatic refresh for all windows ")
         self.send_refresh(-1)
 
-    def _process_hello(self, packet):
-        capabilities = packet[1]
-        self.server_capabilities = capabilities
+    def parse_server_capabilities(self, capabilities):
+        XpraClientBase.parse_server_capabilities(self, capabilities)
         if not self.session_name:
             self.session_name = capabilities.get("session_name", "Xpra")
         try:
@@ -931,10 +937,6 @@ class XpraClient(XpraClientBase):
             glib.set_application_name(self.session_name)
         except ImportError, e:
             log.warn("glib is missing, cannot set the application name, please install glib's python bindings: %s", e)
-        self._remote_version = capabilities.get("version") or capabilities.get("__prerelease_version")
-        if not is_compatible_with(self._remote_version):
-            self.quit()
-            return
         #figure out the maximum actual desktop size and use to
         #calculate the maximum size of a packet (a full screen update packet)
         root_w, root_h = get_root_size()
@@ -974,6 +976,7 @@ class XpraClient(XpraClientBase):
         if e and e!=self.encoding:
             log.debug("server is using %s encoding" % e)
             self.encoding = e
+        self.window_configure = capabilities.get("window_configure", False)
         self.notifications_enabled = capabilities.get("notifications", False)
         clipboard_server_support = capabilities.get("clipboard", True)
         self.clipboard_enabled = clipboard_server_support and self._client_extras.supports_clipboard()
@@ -1014,7 +1017,8 @@ class XpraClient(XpraClientBase):
 
     def set_encoding(self, encoding):
         assert encoding in ENCODINGS
-        assert encoding in self.server_capabilities.get("encodings", [])
+        server_encodings = self.server_capabilities.get("encodings", [])
+        assert encoding in server_encodings, "encoding %s is not supported by the server! (only: %s)" % (encoding, server_encodings)
         self.encoding = encoding
         self.send(["encoding", encoding])
 
@@ -1058,8 +1062,7 @@ class XpraClient(XpraClientBase):
                 data_start = ctypes.c_uint.from_buffer(self.mmap, 0)
                 offset, length = data[-1]
                 data_start.value = offset+length
-        if packet_sequence:
-            self.send_now(["damage-sequence", packet_sequence, wid, width, height, decode_time])
+        self.send_now(["damage-sequence", packet_sequence, wid, width, height, decode_time])
 
     def _process_cursor(self, packet):
         (_, new_cursor) = packet
@@ -1095,9 +1098,10 @@ class XpraClient(XpraClientBase):
         self._client_extras.close_notify(nid)
 
     def _process_window_metadata(self, packet):
-        (wid, metadata) = packet[1:3]
-        window = self._id_to_window[wid]
-        window.update_metadata(metadata)
+        wid, metadata = packet[1:3]
+        window = self._id_to_window.get(wid)
+        if window:
+            window.update_metadata(metadata)
 
     def _process_configure_override_redirect(self, packet):
         (wid, x, y, w, h) = packet[1:6]
