@@ -94,9 +94,10 @@ def untilConcludes(f, *a, **kw):
                 continue
             raise
 
-class RGB24(object):
-    def __init__(self, pixels):
-        self.pixels = pixels
+class Compressible(object):
+    def __init__(self, datatype, data):
+        self.datatype = datatype
+        self.data = data
 
 class Protocol(object):
     CONNECTION_LOST = "connection-lost"
@@ -224,21 +225,25 @@ class Protocol(object):
         ]
         """
         packets = []
-        if self.raw_packets:
-            for i in range(len(packet)):
-                item = packet[i]
-                if type(item)==str and len(item)>=4096:
-                    #add new binary packet with large item:
-                    if sys.version>='3':
-                        item = item.encode("latin1")
-                    packets.append((i, False, item))
-                    #replace this item with an empty string placeholder:
-                    packet[i] = ''
-                elif type(item)==RGB24:
-                    #this is binary, but we *DO* want to compress it since it isn't compressed already!
-                    log("unwrapping %s bytes of rgb24 data", len(item.pixels))
-                    packets.append((i, True, item.pixels))
+        for i in range(len(packet)):
+            item = packet[i]
+            if self.raw_packets and type(item)==str and len(item)>=4096:
+                #add new binary packet with large item:
+                if sys.version>='3':
+                    item = item.encode("latin1")
+                packets.append((i, False, item))
+                #replace this item with an empty string placeholder:
+                packet[i] = ''
+            elif type(item)==Compressible:
+                #this is binary, but we *DO* want to compress it since it isn't compressed already!
+                log("unwrapping %s bytes of %s data", len(item.data), item.datatype)
+                if self.raw_packets:
+                    #make a new compressed packet for it:
+                    packets.append((i, True, item.data))
                     packet[i] = ''
+                else:
+                    #old compression code: just unwrap it in place:
+                    packet[i] = item.data
         #now the main packet (or what is left of it):
         try:
             main_packet = bencode(packet)
@@ -384,8 +389,6 @@ class Protocol(object):
                 else:
                     read_buffer = buf
                 bl = len(read_buffer)
-                if self.max_packet_size>0 and bl>self.max_packet_size:
-                    return self._call_connection_lost("read buffer too big: %s (maximum is %s), dropping this connection!" % (bl, self.max_packet_size))
                 while not self._closed:
                     bl = len(read_buffer)
                     if bl<=0:
@@ -414,6 +417,16 @@ class Protocol(object):
                             read_buffer = read_buffer[8:]
                         bl = len(read_buffer)
 
+                    if current_packet_size>self.max_packet_size:
+                        #this packet is seemingly too big, but check again from the main UI thread
+                        #this gives 'set_max_packet_size' a chance to run
+                        def check_packet_size(size_to_check):
+                            log.info("check_packet_size(%s) limit is %s", size_to_check, self.max_packet_size)
+                            if size_to_check>self.max_packet_size:
+                                return self._call_connection_lost("invalid packet: size requested is %s (maximum allowed is %s), dropping this connection!" %
+                                                              (size_to_check, self.max_packet_size))
+                        gobject.idle_add(check_packet_size, current_packet_size)
+
                     if current_packet_size>0 and bl<current_packet_size:
                         # incomplete packet
                         break
